#include <QApplication>
#include <QDebug>
#include <QWidget>
#include <QMenu>
#include <QPushButton>
#include <QHBoxLayout>

#include <QAction>
#include <QIcon>
#include <QPixmap>
#include <QSystemTrayIcon>
#include <QStyle>
#include <QSysInfo>
#include <algorithm>
#include <iterator>

static void formatTitle(QTextStream &str)
{
    str << QT_VERSION_STR << ' ';
#if QT_VERSION >= 0x050000
    str << QGuiApplication::platformName();
#  if QT_VERSION >= 0x050400
    str << '/' << QSysInfo::prettyProductName();
#  endif
#else
#  if defined(Q_OS_WIN)
    str << "windows";
#  elif defined(Q_OS_MAC)
    str << "macOS";
#  else
    str << "unix";
#  endif
#endif
}

static QString title()
{
    QString r;
    QTextStream str(&r);
    formatTitle(str);
    return r;
}

static QString description(const QWidget *w = 0)
{
#if QT_VERSION >= 0x050000
    qreal dpr = w ? w->devicePixelRatioF() : qApp->devicePixelRatio();
#else
    qreal dpr = 1;
#endif

    QString t;
    QTextStream str(&t);
    formatTitle(str);
    if (!qFuzzyCompare(dpr, qreal(1)))
        str << ", DPR=" << dpr;
    const int dpi = w ? w->logicalDpiX() : 96;
    if (dpi != 96)
        str << ", " << dpi << "DPI";
    str << ", " << (w ? w->style()->objectName() : QApplication::style()->objectName());
    return t;
}

class Widget : public QWidget
{
  public:
    Widget();

private:
    QSystemTrayIcon m_trayIcon;
};

Widget::Widget()
{
    auto trayIconMenu = new QMenu(this);
    trayIconMenu->addAction("Quit", this, &QWidget::close);
    m_trayIcon.setContextMenu(trayIconMenu);

    QPixmap px(32, 32);
    px.fill(Qt::red);
    m_trayIcon.setIcon(QIcon(px));

    auto lt = new QHBoxLayout(this);
    auto sb = new QPushButton("Show", this);
    connect(sb, &QAbstractButton::clicked, &m_trayIcon, &QSystemTrayIcon::show);
    auto hb = new QPushButton("Hide", this);
    connect(hb, &QAbstractButton::clicked, &m_trayIcon, &QSystemTrayIcon::hide);
    lt->addWidget(hb);
    lt->addWidget(sb);
    m_trayIcon.show();
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    QStringList args = app.arguments();
    args.pop_front();

    Widget w;
    qDebug()
#if QT_VERSION >= 0x050400
            .noquote()
#endif
        << description(&w) << args;
    w.setWindowTitle(title());
    w.show();


    return app.exec();
}
